/*
 * CRRCsim - the Charles River Radio Control Club Flight Simulator Project
 *   Copyright (C) 2009 - Jens Wilhelm Wulf (original author)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
#ifndef CNTRL_ATTITUDE_H
# define CNTRL_ATTITUDE_H

#include "../controller.h"

#include <fstream>

/**
 * This controller makes pitch/roll stick inputs control attitude, and yaw 
 * stick input control rotation rate.
 * 
 * @author Jens W. Wulf
 */
class Cntrl_MCopter01 : public Controller
{
public:
  
  Cntrl_MCopter01(SimpleXMLTransfer* cfg);
  
  virtual void Reset();
  
  virtual void Calc(double      dt, 
                    FDMBase*    fdm,
                    TSimInputs* pInputsFromUser,
                    TSimInputs* pInputsToFDM);

  virtual ~Cntrl_MCopter01();
  
private:

  class ControllerA
  {
  private:
    double scale;
    double kp;
    double kd;
  public:
    double GetSetpoint(double in)
    {
      return(in*scale);
    }
    
    void ReadData(SimpleXMLTransfer* cfg)
    {
      scale = cfg->getDouble("scale");
      kp    = cfg->getDouble("kp");
      kd    = cfg->getDouble("kd");
    }
    
    double Step(double dt, double setpoint, double valA, double valR)
    {
      double setpointR = (setpoint*scale - valA) * kp;
      return((setpointR-valR) * kd);
    }
  };
  ControllerA   controllerA[2];
  
  class ControllerYaw
  {
  private:
    double scale;
    double kp;
    double ki;
  public:
    double integrator;
    
    double GetSetpoint(double in)
    {
      return(in*scale);
    }
    
    void ReadData(SimpleXMLTransfer* cfg)
    {
      scale = cfg->getDouble("scale");
      kp    = cfg->getDouble("kp");
      ki    = cfg->getDouble("ki");
    }
    
    double Step(double dt, double setpoint, double val)
    {
      double diff = setpoint*scale - val;
      integrator += diff * dt;
      return(integrator * ki + diff * kp);
    }
  };
  ControllerYaw   controllerYaw;
  
  int   nLog;
  
  std::ofstream outlog;
};

#endif
