/*
 * CRRCsim - the Charles River Radio Control Club Flight Simulator Project
 *
 *  Copyright (C) 2010 Joel Lienard (original author)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
  

/*************
 * crrc_wind_from_terrain.cpp
 * 
 * very simple wind calculation from terain profil
 *
 *********/
 
 /*
 We make a modelling locally the ground by a simple shape.
 That modelling is made from some points (2 - 5).
 The distance from these measurement points  of the terrain is all the bigger as the height is big.
 Mode 1: modelling by a plan
 Mode 2: modelling by horizontal cylinder
 Mode 3: modelling by cylinder inclined
 */
#include "../global.h"
#include <crrc_config.h>
#include "model_based_scenery.h"
#include "../crrc_main.h"
#include "wind_from_terrain.h"

 #define DEEPEST_HELL  -9999.0
 
 
 const int mode = 1; 
 const float COEF= 0.5;//define angle of measure
 
 void wind_from_terrain (double X,double  Y,double  Z,
          float *x_wind, float *y_wind, float *z_wind)
{
   Z = -Z; //positive down -> positive up
   sgVec3 wind;
   float WindVel = cfg->wind->getVelocity();
   float WindDir = cfg->wind->getDirection()*M_PI/180;
   float z_c= Global::scenery->getHeight(X, Y);//Height below the point
   float dz = Z-z_c;
   float dx = dz * COEF * cos(WindDir); 
   float dy = dz * COEF * sin(WindDir);

   float z_f = Global::scenery->getHeight(X+dx, Y+dy);//in front of (in the wind)
   if(z_f==DEEPEST_HELL){ z_f= z_c;}
   //float z_l = Global::scenery->getHeight(X+dx, Y-dsin);//left
   float z_b = Global::scenery->getHeight(X-dx, Y-dy);//Behind (under the wind)
   if(z_b==DEEPEST_HELL){ z_b= z_c;}
   //float z_r = Global::scenery->getHeight(X-dx, Y+dy);//right
   sgVec3 p_c ,p_f, p_b;
   //sgVec3 p_0, p_l, p_r;
   //sgSetVec3(p_0,X,Y,Z);
   sgSetVec3(p_c, X, Y, z_c);
   sgSetVec3(p_f, X+dx, Y+dy, -z_f);
   sgSetVec3(p_b, X-dx, Y-dy, -z_b);

if (mode == 1)
    //
    //We tilt the vector of wind along the slope, with the same speed in module
    //
 {
    sgVec3 dir;
    sgSubVec3  (dir, p_f, p_b);
    sgNormaliseVec3 ( dir );//-> Unit vector in the direction of the wind
    sgScaleVec3  ( wind, dir,-WindVel );
   
 }
else if(mode==2)
{
   //calcul angle et convexite du sol sous le point
    //TODO
}
// return results
   *x_wind = wind[0];
   *y_wind = wind[1];
   *z_wind = wind[2];

#if 0
//for debug 
    *x_wind = -.7 * WindVel * cos(M_PI*cfg->wind->getDirection()/180);
    *y_wind = -.7 * WindVel * sin(M_PI*cfg->wind->getDirection()/180);
    *z_wind = -0.3 * WindVel;
#endif
}

